/*****************************************************************************
 * PICCONV.C
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "picture.h"
#ifdef OS_ATARI
#include <osbind.h>
#endif

static unsigned char const masktab[8] = {
	0xC0, 0x30, 0x0C, 0x03, 0xC0, 0x30, 0x0C, 0x03
};

static TOSPALETTE const palet1 = {
	{ 0x07, 0x77 },
	{ 0x03, 0x33 },
	{ 0x03, 0x33 },
	{ 0x04, 0x44 },
	{ 0x03, 0x33 },
	{ 0x04, 0x44 },
	{ 0x04, 0x44 },
	{ 0x06, 0x66 },
	{ 0x03, 0x33 },
	{ 0x04, 0x44 },
	{ 0x04, 0x44 },
	{ 0x06, 0x66 },
	{ 0x04, 0x44 },
	{ 0x06, 0x66 },
	{ 0x06, 0x66 },
	{ 0x00, 0x00 }
};

PALETTE const std256_palette = {
	{ 0xff, 0xff, 0xff }, /*   0 == WHITE (0) */
	{ 0xff, 0x00, 0x00 }, /*   1 == RED (2) */
	{ 0x00, 0xff, 0x00 }, /*   2 == GREEN (3) */
	{ 0xff, 0xff, 0x00 }, /*   3 == YELLOW (6) */
	{ 0x00, 0x00, 0xff }, /*   4 == BLUE (4) */
	{ 0xff, 0x00, 0xff }, /*   5 == MAGENTA (7) */
	{ 0x00, 0xff, 0xff }, /*   6 == CYAN (5) */
	{ 0xcc, 0xcc, 0xcc }, /*   7 == LWHITE (8) */
	{ 0x88, 0x88, 0x88 }, /*   8 == LBLACK (9) */
	{ 0x88, 0x00, 0x00 }, /*   9 == LRED (10) */
	{ 0x00, 0x88, 0x00 }, /*  10 == LGREEN (11) */
	{ 0x88, 0x88, 0x00 }, /*  11 == LYELLOW (14) */
	{ 0x00, 0x00, 0x88 }, /*  12 == LBLUE (12) */
	{ 0x88, 0x00, 0x88 }, /*  13 == LMAGENTA (15) */
	{ 0x00, 0x88, 0x88 }, /*  14 == LCYAN (13) */
	{ 0x00, 0x00, 0x00 }, /*  15 == BLACK (255) */
	{ 0xfd, 0xfd, 0xfd }, /*  16 == 16 */
	{ 0xec, 0xec, 0xec }, /*  17 == 17 */
	{ 0xdb, 0xdb, 0xdb }, /*  18 == 18 */
	{ 0xca, 0xca, 0xca }, /*  19 == 19 */
	{ 0xb9, 0xb9, 0xb9 }, /*  20 == 20 */
	{ 0xa8, 0xa8, 0xa8 }, /*  21 == 21 */
	{ 0x97, 0x97, 0x97 }, /*  22 == 22 */
	{ 0x87, 0x87, 0x87 }, /*  23 == 23 */
	{ 0x76, 0x76, 0x76 }, /*  24 == 24 */
	{ 0x65, 0x65, 0x65 }, /*  25 == 25 */
	{ 0x54, 0x54, 0x54 }, /*  26 == 26 */
	{ 0x43, 0x43, 0x43 }, /*  27 == 27 */
	{ 0x32, 0x32, 0x32 }, /*  28 == 28 */
	{ 0x21, 0x21, 0x21 }, /*  29 == 29 */
	{ 0x10, 0x10, 0x10 }, /*  30 == 30 */
	{ 0x00, 0x00, 0x00 }, /*  31 == 31 */
	{ 0xfd, 0x00, 0x00 }, /*  32 == 32 */
	{ 0xfd, 0x00, 0x10 }, /*  33 == 33 */
	{ 0xfd, 0x00, 0x21 }, /*  34 == 34 */
	{ 0xfd, 0x00, 0x32 }, /*  35 == 35 */
	{ 0xfd, 0x00, 0x43 }, /*  36 == 36 */
	{ 0xfd, 0x00, 0x54 }, /*  37 == 37 */
	{ 0xfd, 0x00, 0x65 }, /*  38 == 38 */
	{ 0xfd, 0x00, 0x76 }, /*  39 == 39 */
	{ 0xfd, 0x00, 0x87 }, /*  40 == 40 */
	{ 0xfd, 0x00, 0x97 }, /*  41 == 41 */
	{ 0xfd, 0x00, 0xa8 }, /*  42 == 42 */
	{ 0xfd, 0x00, 0xb9 }, /*  43 == 43 */
	{ 0xfd, 0x00, 0xca }, /*  44 == 44 */
	{ 0xfd, 0x00, 0xdb }, /*  45 == 45 */
	{ 0xfd, 0x00, 0xec }, /*  46 == 46 */
	{ 0xfd, 0x00, 0xfd }, /*  47 == 47 */
	{ 0xec, 0x00, 0xfd }, /*  48 == 48 */
	{ 0xdb, 0x00, 0xfd }, /*  49 == 49 */
	{ 0xca, 0x00, 0xfd }, /*  50 == 50 */
	{ 0xb9, 0x00, 0xfd }, /*  51 == 51 */
	{ 0xa8, 0x00, 0xfd }, /*  52 == 52 */
	{ 0x97, 0x00, 0xfd }, /*  53 == 53 */
	{ 0x87, 0x00, 0xfd }, /*  54 == 54 */
	{ 0x76, 0x00, 0xfd }, /*  55 == 55 */
	{ 0x65, 0x00, 0xfd }, /*  56 == 56 */
	{ 0x54, 0x00, 0xfd }, /*  57 == 57 */
	{ 0x43, 0x00, 0xfd }, /*  58 == 58 */
	{ 0x32, 0x00, 0xfd }, /*  59 == 59 */
	{ 0x21, 0x00, 0xfd }, /*  60 == 60 */
	{ 0x10, 0x00, 0xfd }, /*  61 == 61 */
	{ 0x00, 0x00, 0xfd }, /*  62 == 62 */
	{ 0x00, 0x10, 0xfd }, /*  63 == 63 */
	{ 0x00, 0x21, 0xfd }, /*  64 == 64 */
	{ 0x00, 0x32, 0xfd }, /*  65 == 65 */
	{ 0x00, 0x43, 0xfd }, /*  66 == 66 */
	{ 0x00, 0x54, 0xfd }, /*  67 == 67 */
	{ 0x00, 0x65, 0xfd }, /*  68 == 68 */
	{ 0x00, 0x76, 0xfd }, /*  69 == 69 */
	{ 0x00, 0x87, 0xfd }, /*  70 == 70 */
	{ 0x00, 0x97, 0xfd }, /*  71 == 71 */
	{ 0x00, 0xa8, 0xfd }, /*  72 == 72 */
	{ 0x00, 0xb9, 0xfd }, /*  73 == 73 */
	{ 0x00, 0xca, 0xfd }, /*  74 == 74 */
	{ 0x00, 0xdb, 0xfd }, /*  75 == 75 */
	{ 0x00, 0xec, 0xfd }, /*  76 == 76 */
	{ 0x00, 0xfd, 0xfd }, /*  77 == 77 */
	{ 0x00, 0xfd, 0xec }, /*  78 == 78 */
	{ 0x00, 0xfd, 0xdb }, /*  79 == 79 */
	{ 0x00, 0xfd, 0xca }, /*  80 == 80 */
	{ 0x00, 0xfd, 0xb9 }, /*  81 == 81 */
	{ 0x00, 0xfd, 0xa8 }, /*  82 == 82 */
	{ 0x00, 0xfd, 0x97 }, /*  83 == 83 */
	{ 0x00, 0xfd, 0x87 }, /*  84 == 84 */
	{ 0x00, 0xfd, 0x76 }, /*  85 == 85 */
	{ 0x00, 0xfd, 0x65 }, /*  86 == 86 */
	{ 0x00, 0xfd, 0x54 }, /*  87 == 87 */
	{ 0x00, 0xfd, 0x43 }, /*  88 == 88 */
	{ 0x00, 0xfd, 0x32 }, /*  89 == 89 */
	{ 0x00, 0xfd, 0x21 }, /*  90 == 90 */
	{ 0x00, 0xfd, 0x10 }, /*  91 == 91 */
	{ 0x00, 0xfd, 0x00 }, /*  92 == 92 */
	{ 0x10, 0xfd, 0x00 }, /*  93 == 93 */
	{ 0x21, 0xfd, 0x00 }, /*  94 == 94 */
	{ 0x32, 0xfd, 0x00 }, /*  95 == 95 */
	{ 0x43, 0xfd, 0x00 }, /*  96 == 96 */
	{ 0x54, 0xfd, 0x00 }, /*  97 == 97 */
	{ 0x65, 0xfd, 0x00 }, /*  98 == 98 */
	{ 0x76, 0xfd, 0x00 }, /*  99 == 99 */
	{ 0x87, 0xfd, 0x00 }, /* 100 == 100 */
	{ 0x97, 0xfd, 0x00 }, /* 101 == 101 */
	{ 0xa8, 0xfd, 0x00 }, /* 102 == 102 */
	{ 0xb9, 0xfd, 0x00 }, /* 103 == 103 */
	{ 0xca, 0xfd, 0x00 }, /* 104 == 104 */
	{ 0xdb, 0xfd, 0x00 }, /* 105 == 105 */
	{ 0xec, 0xfd, 0x00 }, /* 106 == 106 */
	{ 0xfd, 0xfd, 0x00 }, /* 107 == 107 */
	{ 0xfd, 0xec, 0x00 }, /* 108 == 108 */
	{ 0xfd, 0xdb, 0x00 }, /* 109 == 109 */
	{ 0xfd, 0xca, 0x00 }, /* 110 == 110 */
	{ 0xfd, 0xb9, 0x00 }, /* 111 == 111 */
	{ 0xfd, 0xa8, 0x00 }, /* 112 == 112 */
	{ 0xfd, 0x97, 0x00 }, /* 113 == 113 */
	{ 0xfd, 0x87, 0x00 }, /* 114 == 114 */
	{ 0xfd, 0x76, 0x00 }, /* 115 == 115 */
	{ 0xfd, 0x65, 0x00 }, /* 116 == 116 */
	{ 0xfd, 0x54, 0x00 }, /* 117 == 117 */
	{ 0xfd, 0x43, 0x00 }, /* 118 == 118 */
	{ 0xfd, 0x32, 0x00 }, /* 119 == 119 */
	{ 0xfd, 0x21, 0x00 }, /* 120 == 120 */
	{ 0xfd, 0x10, 0x00 }, /* 121 == 121 */
	{ 0xb9, 0x00, 0x00 }, /* 122 == 122 */
	{ 0xb9, 0x00, 0x10 }, /* 123 == 123 */
	{ 0xb9, 0x00, 0x21 }, /* 124 == 124 */
	{ 0xb9, 0x00, 0x32 }, /* 125 == 125 */
	{ 0xb9, 0x00, 0x43 }, /* 126 == 126 */
	{ 0xb9, 0x00, 0x54 }, /* 127 == 127 */
	{ 0xb9, 0x00, 0x65 }, /* 128 == 128 */
	{ 0xb9, 0x00, 0x76 }, /* 129 == 129 */
	{ 0xb9, 0x00, 0x87 }, /* 130 == 130 */
	{ 0xb9, 0x00, 0x97 }, /* 131 == 131 */
	{ 0xb9, 0x00, 0xa8 }, /* 132 == 132 */
	{ 0xb9, 0x00, 0xb9 }, /* 133 == 133 */
	{ 0xa8, 0x00, 0xb9 }, /* 134 == 134 */
	{ 0x97, 0x00, 0xb9 }, /* 135 == 135 */
	{ 0x87, 0x00, 0xb9 }, /* 136 == 136 */
	{ 0x76, 0x00, 0xb9 }, /* 137 == 137 */
	{ 0x65, 0x00, 0xb9 }, /* 138 == 138 */
	{ 0x54, 0x00, 0xb9 }, /* 139 == 139 */
	{ 0x43, 0x00, 0xb9 }, /* 140 == 140 */
	{ 0x32, 0x00, 0xb9 }, /* 141 == 141 */
	{ 0x21, 0x00, 0xb9 }, /* 142 == 142 */
	{ 0x10, 0x00, 0xb9 }, /* 143 == 143 */
	{ 0x00, 0x00, 0xb9 }, /* 144 == 144 */
	{ 0x00, 0x10, 0xb9 }, /* 145 == 145 */
	{ 0x00, 0x21, 0xb9 }, /* 146 == 146 */
	{ 0x00, 0x32, 0xb9 }, /* 147 == 147 */
	{ 0x00, 0x43, 0xb9 }, /* 148 == 148 */
	{ 0x00, 0x54, 0xb9 }, /* 149 == 149 */
	{ 0x00, 0x65, 0xb9 }, /* 150 == 150 */
	{ 0x00, 0x76, 0xb9 }, /* 151 == 151 */
	{ 0x00, 0x87, 0xb9 }, /* 152 == 152 */
	{ 0x00, 0x97, 0xb9 }, /* 153 == 153 */
	{ 0x00, 0xa8, 0xb9 }, /* 154 == 154 */
	{ 0x00, 0xb9, 0xb9 }, /* 155 == 155 */
	{ 0x00, 0xb9, 0xa8 }, /* 156 == 156 */
	{ 0x00, 0xb9, 0x97 }, /* 157 == 157 */
	{ 0x00, 0xb9, 0x87 }, /* 158 == 158 */
	{ 0x00, 0xb9, 0x76 }, /* 159 == 159 */
	{ 0x00, 0xb9, 0x65 }, /* 160 == 160 */
	{ 0x00, 0xb9, 0x54 }, /* 161 == 161 */
	{ 0x00, 0xb9, 0x43 }, /* 162 == 162 */
	{ 0x00, 0xb9, 0x32 }, /* 163 == 163 */
	{ 0x00, 0xb9, 0x21 }, /* 164 == 164 */
	{ 0x00, 0xb9, 0x10 }, /* 165 == 165 */
	{ 0x00, 0xb9, 0x00 }, /* 166 == 166 */
	{ 0x10, 0xb9, 0x00 }, /* 167 == 167 */
	{ 0x21, 0xb9, 0x00 }, /* 168 == 168 */
	{ 0x32, 0xb9, 0x00 }, /* 169 == 169 */
	{ 0x43, 0xb9, 0x00 }, /* 170 == 170 */
	{ 0x54, 0xb9, 0x00 }, /* 171 == 171 */
	{ 0x65, 0xb9, 0x00 }, /* 172 == 172 */
	{ 0x76, 0xb9, 0x00 }, /* 173 == 173 */
	{ 0x87, 0xb9, 0x00 }, /* 174 == 174 */
	{ 0x97, 0xb9, 0x00 }, /* 175 == 175 */
	{ 0xa8, 0xb9, 0x00 }, /* 176 == 176 */
	{ 0xb9, 0xb9, 0x00 }, /* 177 == 177 */
	{ 0xb9, 0xa8, 0x00 }, /* 178 == 178 */
	{ 0xb9, 0x97, 0x00 }, /* 179 == 179 */
	{ 0xb9, 0x87, 0x00 }, /* 180 == 180 */
	{ 0xb9, 0x76, 0x00 }, /* 181 == 181 */
	{ 0xb9, 0x65, 0x00 }, /* 182 == 182 */
	{ 0xb9, 0x54, 0x00 }, /* 183 == 183 */
	{ 0xb9, 0x43, 0x00 }, /* 184 == 184 */
	{ 0xb9, 0x32, 0x00 }, /* 185 == 185 */
	{ 0xb9, 0x21, 0x00 }, /* 186 == 186 */
	{ 0xb9, 0x10, 0x00 }, /* 187 == 187 */
	{ 0x76, 0x00, 0x00 }, /* 188 == 188 */
	{ 0x76, 0x00, 0x10 }, /* 189 == 189 */
	{ 0x76, 0x00, 0x21 }, /* 190 == 190 */
	{ 0x76, 0x00, 0x32 }, /* 191 == 191 */
	{ 0x76, 0x00, 0x43 }, /* 192 == 192 */
	{ 0x76, 0x00, 0x54 }, /* 193 == 193 */
	{ 0x76, 0x00, 0x65 }, /* 194 == 194 */
	{ 0x76, 0x00, 0x76 }, /* 195 == 195 */
	{ 0x65, 0x00, 0x76 }, /* 196 == 196 */
	{ 0x54, 0x00, 0x76 }, /* 197 == 197 */
	{ 0x43, 0x00, 0x76 }, /* 198 == 198 */
	{ 0x32, 0x00, 0x76 }, /* 199 == 199 */
	{ 0x21, 0x00, 0x76 }, /* 200 == 200 */
	{ 0x10, 0x00, 0x76 }, /* 201 == 201 */
	{ 0x00, 0x00, 0x76 }, /* 202 == 202 */
	{ 0x00, 0x10, 0x76 }, /* 203 == 203 */
	{ 0x00, 0x21, 0x76 }, /* 204 == 204 */
	{ 0x00, 0x32, 0x76 }, /* 205 == 205 */
	{ 0x00, 0x43, 0x76 }, /* 206 == 206 */
	{ 0x00, 0x54, 0x76 }, /* 207 == 207 */
	{ 0x00, 0x65, 0x76 }, /* 208 == 208 */
	{ 0x00, 0x76, 0x76 }, /* 209 == 209 */
	{ 0x00, 0x76, 0x65 }, /* 210 == 210 */
	{ 0x00, 0x76, 0x54 }, /* 211 == 211 */
	{ 0x00, 0x76, 0x43 }, /* 212 == 212 */
	{ 0x00, 0x76, 0x32 }, /* 213 == 213 */
	{ 0x00, 0x76, 0x21 }, /* 214 == 214 */
	{ 0x00, 0x76, 0x10 }, /* 215 == 215 */
	{ 0x00, 0x76, 0x00 }, /* 216 == 216 */
	{ 0x10, 0x76, 0x00 }, /* 217 == 217 */
	{ 0x21, 0x76, 0x00 }, /* 218 == 218 */
	{ 0x32, 0x76, 0x00 }, /* 219 == 219 */
	{ 0x43, 0x76, 0x00 }, /* 220 == 220 */
	{ 0x54, 0x76, 0x00 }, /* 221 == 221 */
	{ 0x65, 0x76, 0x00 }, /* 222 == 222 */
	{ 0x76, 0x76, 0x00 }, /* 223 == 223 */
	{ 0x76, 0x65, 0x00 }, /* 224 == 224 */
	{ 0x76, 0x54, 0x00 }, /* 225 == 225 */
	{ 0x76, 0x43, 0x00 }, /* 226 == 226 */
	{ 0x76, 0x32, 0x00 }, /* 227 == 227 */
	{ 0x76, 0x21, 0x00 }, /* 228 == 228 */
	{ 0x76, 0x10, 0x00 }, /* 229 == 229 */
	{ 0x43, 0x00, 0x00 }, /* 230 == 230 */
	{ 0x43, 0x00, 0x10 }, /* 231 == 231 */
	{ 0x43, 0x00, 0x21 }, /* 232 == 232 */
	{ 0x43, 0x00, 0x32 }, /* 233 == 233 */
	{ 0x43, 0x00, 0x43 }, /* 234 == 234 */
	{ 0x32, 0x00, 0x43 }, /* 235 == 235 */
	{ 0x21, 0x00, 0x43 }, /* 236 == 236 */
	{ 0x10, 0x00, 0x43 }, /* 237 == 237 */
	{ 0x00, 0x00, 0x43 }, /* 238 == 238 */
	{ 0x00, 0x10, 0x43 }, /* 239 == 239 */
	{ 0x00, 0x21, 0x43 }, /* 240 == 240 */
	{ 0x00, 0x32, 0x43 }, /* 241 == 241 */
	{ 0x00, 0x43, 0x43 }, /* 242 == 242 */
	{ 0x00, 0x43, 0x32 }, /* 243 == 243 */
	{ 0x00, 0x43, 0x21 }, /* 244 == 244 */
	{ 0x00, 0x43, 0x10 }, /* 245 == 245 */
	{ 0x00, 0x43, 0x00 }, /* 246 == 246 */
	{ 0x10, 0x43, 0x00 }, /* 247 == 247 */
	{ 0x21, 0x43, 0x00 }, /* 248 == 248 */
	{ 0x32, 0x43, 0x00 }, /* 249 == 249 */
	{ 0x43, 0x43, 0x00 }, /* 250 == 250 */
	{ 0x43, 0x32, 0x00 }, /* 251 == 251 */
	{ 0x43, 0x21, 0x00 }, /* 252 == 252 */
	{ 0x43, 0x10, 0x00 }, /* 253 == 253 */
	{ 0xfd, 0xfd, 0xfd }, /* 254 == 254 */
	{ 0x00, 0x00, 0x00 }  /* 255 == BLACK (1) */
};


static unsigned char const pattern1[16 * 16] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x88, 0x00, 0x00, 0x00, 0x88, 0x00, 0x00, 0x00,
	0x88, 0x00, 0x00, 0x00, 0x88, 0x00, 0x00, 0x00,
	0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00,
	0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00,
	0x88, 0x00, 0xAA, 0x00, 0x88, 0x00, 0xAA, 0x00,
	0x88, 0x00, 0xAA, 0x00, 0x88, 0x00, 0xAA, 0x00,
	0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00,
	0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00,
	0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44,
	0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44,
	0xAA, 0x44, 0xAA, 0x11, 0xAA, 0x44, 0xAA, 0x11,
	0xAA, 0x44, 0xAA, 0x11, 0xAA, 0x44, 0xAA, 0x11,
	0xAA, 0x44, 0xAA, 0x55, 0xAA, 0x44, 0xAA, 0x55,
	0xAA, 0x44, 0xAA, 0x55, 0xAA, 0x44, 0xAA, 0x55,
	0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55,
	0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55,
	0x55, 0xBB, 0x55, 0xEE, 0x55, 0xBB, 0x55, 0xEE,
	0x55, 0xBB, 0x55, 0xEE, 0x55, 0xBB, 0x55, 0xEE,
	0x55, 0xBB, 0x55, 0xFF, 0x55, 0xBB, 0x55, 0xFF,
	0x55, 0xBB, 0x55, 0xFF, 0x55, 0xBB, 0x55, 0xFF,
	0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF,
	0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF,
	0x77, 0xFF, 0x55, 0xFF, 0x77, 0xFF, 0x55, 0xFF,
	0x77, 0xFF, 0x55, 0xFF, 0x77, 0xFF, 0x55, 0xFF,
	0x77, 0xFF, 0xDD, 0xFF, 0x77, 0xFF, 0xDD, 0xFF,
	0x77, 0xFF, 0xDD, 0xFF, 0x77, 0xFF, 0xDD, 0xFF,
	0x77, 0xFF, 0xFF, 0xFF, 0x77, 0xFF, 0xFF, 0xFF,
	0x77, 0xFF, 0xFF, 0xFF, 0x77, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};

static unsigned char const pattern2[16 * 16] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x88, 0x00, 0x00, 0x00, 0x88, 0x00, 0x00, 0x00,
	0x88, 0x00, 0x00, 0x00, 0x88, 0x00, 0x00, 0x00,
	0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00,
	0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00,
	0x88, 0x00, 0xAA, 0x00, 0x88, 0x00, 0xAA, 0x00,
	0x88, 0x00, 0xAA, 0x00, 0x88, 0x00, 0xAA, 0x00,
	0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00,
	0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00,
	0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44,
	0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44, 0xAA, 0x44,
	0xAA, 0x44, 0xAA, 0x11, 0xAA, 0x44, 0xAA, 0x11,
	0xAA, 0x44, 0xAA, 0x11, 0xAA, 0x44, 0xAA, 0x11,
	0xAA, 0x44, 0xAA, 0x55, 0xAA, 0x44, 0xAA, 0x55,
	0xAA, 0x44, 0xAA, 0x55, 0xAA, 0x44, 0xAA, 0x55,
	0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55,
	0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55,
	0x55, 0xBB, 0x55, 0xEE, 0x55, 0xBB, 0x55, 0xEE,
	0x55, 0xBB, 0x55, 0xEE, 0x55, 0xBB, 0x55, 0xEE,
	0x55, 0xBB, 0x55, 0xFF, 0x55, 0xBB, 0x55, 0xFF,
	0x55, 0xBB, 0x55, 0xFF, 0x55, 0xBB, 0x55, 0xFF,
	0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF,
	0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF, 0x55, 0xFF,
	0x77, 0xFF, 0x55, 0xFF, 0x77, 0xFF, 0x55, 0xFF,
	0x77, 0xFF, 0x55, 0xFF, 0x77, 0xFF, 0x55, 0xFF,
	0x77, 0xFF, 0xDD, 0xFF, 0x77, 0xFF, 0xDD, 0xFF,
	0x77, 0xFF, 0xDD, 0xFF, 0x77, 0xFF, 0xDD, 0xFF,
	0x77, 0xFF, 0xFF, 0xFF, 0x77, 0xFF, 0xFF, 0xFF,
	0x77, 0xFF, 0xFF, 0xFF, 0x77, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};
static const unsigned char *patptr1[16];
static const unsigned char *patptr2[16];

#define toword(pixels) (((pixels) + 15) >> 4)
#define tobyte(pixels) (toword(pixels) << 1)

/*
 * Map rgb color range [0..255] to vdi color range [0..1000]
 */
static _WORD const rgb_to_vdi_tab[256] = {
   0,    4,    8,   12,   16,   20,   24,   28,   32,   36,   40,   43,   47,   51,   55,   59,
  63,   67,   71,   75,   79,   83,   86,   90,   94,   98,  102,  106,  110,  114,  118,  122,
 126,  129,  133,  137,  141,  145,  149,  153,  157,  161,  165,  168,  172,  176,  180,  184,
 188,  192,  196,  200,  204,  208,  211,  215,  219,  223,  227,  231,  235,  239,  243,  247,
 251,  254,  258,  262,  266,  270,  274,  278,  282,  286,  290,  294,  298,  301,  305,  309,
 313,  317,  321,  325,  329,  333,  337,  341,  345,  349,  352,  356,  360,  364,  368,  372,
 376,  380,  384,  388,  392,  396,  400,  403,  407,  411,  415,  419,  423,  427,  431,  435,
 439,  443,  447,  450,  454,  458,  462,  466,  470,  474,  478,  482,  486,  490,  494,  498,
 501,  505,  509,  513,  517,  521,  525,  529,  533,  537,  541,  545,  549,  552,  556,  560,
 564,  568,  572,  576,  580,  584,  588,  592,  596,  600,  603,  607,  611,  615,  619,  623,
 627,  631,  635,  639,  643,  647,  650,  654,  658,  662,  666,  670,  674,  678,  682,  686,
 690,  694,  698,  701,  705,  709,  713,  717,  721,  725,  729,  733,  737,  741,  745,  749,
 752,  756,  760,  764,  768,  772,  776,  780,  784,  788,  792,  796,  800,  803,  807,  811,
 815,  819,  823,  827,  831,  835,  839,  843,  847,  850,  854,  858,  862,  866,  870,  874,
 878,  882,  886,  890,  894,  898,  901,  905,  909,  913,  917,  921,  925,  929,  933,  937,
 941,  945,  949,  952,  956,  960,  964,  968,  972,  976,  980,  984,  988,  992,  996, 1000
};

/*
 * Map vdi color range [0..1000] to rgb color range [0..255]
 *
 * r = ((color * 25599u) / 100000lu)
 */
static unsigned char const vdi_to_rgb_tab[1000 + 1] = {
	0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01,
	0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x03,
	0x04, 0x04, 0x04, 0x04, 0x05, 0x05, 0x05, 0x05,
	0x06, 0x06, 0x06, 0x06, 0x07, 0x07, 0x07, 0x07,
	0x08, 0x08, 0x08, 0x08, 0x09, 0x09, 0x09, 0x09,
	0x0a, 0x0a, 0x0a, 0x0b, 0x0b, 0x0b, 0x0b, 0x0c,
	0x0c, 0x0c, 0x0c, 0x0d, 0x0d, 0x0d, 0x0d, 0x0e,
	0x0e, 0x0e, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x10,
	0x10, 0x10, 0x10, 0x11, 0x11, 0x11, 0x11, 0x12,
	0x12, 0x12, 0x12, 0x13, 0x13, 0x13, 0x13, 0x14,
	0x14, 0x14, 0x14, 0x15, 0x15, 0x15, 0x16, 0x16,
	0x16, 0x16, 0x17, 0x17, 0x17, 0x17, 0x18, 0x18,
	0x18, 0x18, 0x19, 0x19, 0x19, 0x19, 0x1a, 0x1a,
	0x1a, 0x1a, 0x1b, 0x1b, 0x1b, 0x1b, 0x1c, 0x1c,
	0x1c, 0x1c, 0x1d, 0x1d, 0x1d, 0x1d, 0x1e, 0x1e,
	0x1e, 0x1e, 0x1f, 0x1f, 0x1f, 0x1f, 0x20, 0x20,
	0x20, 0x21, 0x21, 0x21, 0x21, 0x22, 0x22, 0x22,
	0x22, 0x23, 0x23, 0x23, 0x23, 0x24, 0x24, 0x24,
	0x24, 0x25, 0x25, 0x25, 0x25, 0x26, 0x26, 0x26,
	0x26, 0x27, 0x27, 0x27, 0x27, 0x28, 0x28, 0x28,
	0x28, 0x29, 0x29, 0x29, 0x29, 0x2a, 0x2a, 0x2a,
	0x2b, 0x2b, 0x2b, 0x2b, 0x2c, 0x2c, 0x2c, 0x2c,
	0x2d, 0x2d, 0x2d, 0x2d, 0x2e, 0x2e, 0x2e, 0x2e,
	0x2f, 0x2f, 0x2f, 0x2f, 0x30, 0x30, 0x30, 0x30,
	0x31, 0x31, 0x31, 0x31, 0x32, 0x32, 0x32, 0x32,
	0x33, 0x33, 0x33, 0x33, 0x34, 0x34, 0x34, 0x34,
	0x35, 0x35, 0x35, 0x36, 0x36, 0x36, 0x36, 0x37,
	0x37, 0x37, 0x37, 0x38, 0x38, 0x38, 0x38, 0x39,
	0x39, 0x39, 0x39, 0x3a, 0x3a, 0x3a, 0x3a, 0x3b,
	0x3b, 0x3b, 0x3b, 0x3c, 0x3c, 0x3c, 0x3c, 0x3d,
	0x3d, 0x3d, 0x3d, 0x3e, 0x3e, 0x3e, 0x3e, 0x3f,
	0x3f, 0x3f, 0x3f, 0x40, 0x40, 0x40, 0x41, 0x41,
	0x41, 0x41, 0x42, 0x42, 0x42, 0x42, 0x43, 0x43,
	0x43, 0x43, 0x44, 0x44, 0x44, 0x44, 0x45, 0x45,
	0x45, 0x45, 0x46, 0x46, 0x46, 0x46, 0x47, 0x47,
	0x47, 0x47, 0x48, 0x48, 0x48, 0x48, 0x49, 0x49,
	0x49, 0x49, 0x4a, 0x4a, 0x4a, 0x4b, 0x4b, 0x4b,
	0x4b, 0x4c, 0x4c, 0x4c, 0x4c, 0x4d, 0x4d, 0x4d,
	0x4d, 0x4e, 0x4e, 0x4e, 0x4e, 0x4f, 0x4f, 0x4f,
	0x4f, 0x50, 0x50, 0x50, 0x50, 0x51, 0x51, 0x51,
	0x51, 0x52, 0x52, 0x52, 0x52, 0x53, 0x53, 0x53,
	0x53, 0x54, 0x54, 0x54, 0x54, 0x55, 0x55, 0x55,
	0x56, 0x56, 0x56, 0x56, 0x57, 0x57, 0x57, 0x57,
	0x58, 0x58, 0x58, 0x58, 0x59, 0x59, 0x59, 0x59,
	0x5a, 0x5a, 0x5a, 0x5a, 0x5b, 0x5b, 0x5b, 0x5b,
	0x5c, 0x5c, 0x5c, 0x5c, 0x5d, 0x5d, 0x5d, 0x5d,
	0x5e, 0x5e, 0x5e, 0x5e, 0x5f, 0x5f, 0x5f, 0x5f,
	0x60, 0x60, 0x60, 0x61, 0x61, 0x61, 0x61, 0x62,
	0x62, 0x62, 0x62, 0x63, 0x63, 0x63, 0x63, 0x64,
	0x64, 0x64, 0x64, 0x65, 0x65, 0x65, 0x65, 0x66,
	0x66, 0x66, 0x66, 0x67, 0x67, 0x67, 0x67, 0x68,
	0x68, 0x68, 0x68, 0x69, 0x69, 0x69, 0x69, 0x6a,
	0x6a, 0x6a, 0x6b, 0x6b, 0x6b, 0x6b, 0x6c, 0x6c,
	0x6c, 0x6c, 0x6d, 0x6d, 0x6d, 0x6d, 0x6e, 0x6e,
	0x6e, 0x6e, 0x6f, 0x6f, 0x6f, 0x6f, 0x70, 0x70,
	0x70, 0x70, 0x71, 0x71, 0x71, 0x71, 0x72, 0x72,
	0x72, 0x72, 0x73, 0x73, 0x73, 0x73, 0x74, 0x74,
	0x74, 0x74, 0x75, 0x75, 0x75, 0x76, 0x76, 0x76,
	0x76, 0x77, 0x77, 0x77, 0x77, 0x78, 0x78, 0x78,
	0x78, 0x79, 0x79, 0x79, 0x79, 0x7a, 0x7a, 0x7a,
	0x7a, 0x7b, 0x7b, 0x7b, 0x7b, 0x7c, 0x7c, 0x7c,
	0x7c, 0x7d, 0x7d, 0x7d, 0x7d, 0x7e, 0x7e, 0x7e,
	0x7e, 0x7f, 0x7f, 0x7f, 0x7f, 0x80, 0x80, 0x80,
	0x81, 0x81, 0x81, 0x81, 0x82, 0x82, 0x82, 0x82,
	0x83, 0x83, 0x83, 0x83, 0x84, 0x84, 0x84, 0x84,
	0x85, 0x85, 0x85, 0x85, 0x86, 0x86, 0x86, 0x86,
	0x87, 0x87, 0x87, 0x87, 0x88, 0x88, 0x88, 0x88,
	0x89, 0x89, 0x89, 0x89, 0x8a, 0x8a, 0x8a, 0x8b,
	0x8b, 0x8b, 0x8b, 0x8c, 0x8c, 0x8c, 0x8c, 0x8d,
	0x8d, 0x8d, 0x8d, 0x8e, 0x8e, 0x8e, 0x8e, 0x8f,
	0x8f, 0x8f, 0x8f, 0x90, 0x90, 0x90, 0x90, 0x91,
	0x91, 0x91, 0x91, 0x92, 0x92, 0x92, 0x92, 0x93,
	0x93, 0x93, 0x93, 0x94, 0x94, 0x94, 0x94, 0x95,
	0x95, 0x95, 0x96, 0x96, 0x96, 0x96, 0x97, 0x97,
	0x97, 0x97, 0x98, 0x98, 0x98, 0x98, 0x99, 0x99,
	0x99, 0x99, 0x9a, 0x9a, 0x9a, 0x9a, 0x9b, 0x9b,
	0x9b, 0x9b, 0x9c, 0x9c, 0x9c, 0x9c, 0x9d, 0x9d,
	0x9d, 0x9d, 0x9e, 0x9e, 0x9e, 0x9e, 0x9f, 0x9f,
	0x9f, 0x9f, 0xa0, 0xa0, 0xa0, 0xa1, 0xa1, 0xa1,
	0xa1, 0xa2, 0xa2, 0xa2, 0xa2, 0xa3, 0xa3, 0xa3,
	0xa3, 0xa4, 0xa4, 0xa4, 0xa4, 0xa5, 0xa5, 0xa5,
	0xa5, 0xa6, 0xa6, 0xa6, 0xa6, 0xa7, 0xa7, 0xa7,
	0xa7, 0xa8, 0xa8, 0xa8, 0xa8, 0xa9, 0xa9, 0xa9,
	0xa9, 0xaa, 0xaa, 0xaa, 0xab, 0xab, 0xab, 0xab,
	0xac, 0xac, 0xac, 0xac, 0xad, 0xad, 0xad, 0xad,
	0xae, 0xae, 0xae, 0xae, 0xaf, 0xaf, 0xaf, 0xaf,
	0xb0, 0xb0, 0xb0, 0xb0, 0xb1, 0xb1, 0xb1, 0xb1,
	0xb2, 0xb2, 0xb2, 0xb2, 0xb3, 0xb3, 0xb3, 0xb3,
	0xb4, 0xb4, 0xb4, 0xb4, 0xb5, 0xb5, 0xb5, 0xb6,
	0xb6, 0xb6, 0xb6, 0xb7, 0xb7, 0xb7, 0xb7, 0xb8,
	0xb8, 0xb8, 0xb8, 0xb9, 0xb9, 0xb9, 0xb9, 0xba,
	0xba, 0xba, 0xba, 0xbb, 0xbb, 0xbb, 0xbb, 0xbc,
	0xbc, 0xbc, 0xbc, 0xbd, 0xbd, 0xbd, 0xbd, 0xbe,
	0xbe, 0xbe, 0xbe, 0xbf, 0xbf, 0xbf, 0xbf, 0xc0,
	0xc0, 0xc0, 0xc1, 0xc1, 0xc1, 0xc1, 0xc2, 0xc2,
	0xc2, 0xc2, 0xc3, 0xc3, 0xc3, 0xc3, 0xc4, 0xc4,
	0xc4, 0xc4, 0xc5, 0xc5, 0xc5, 0xc5, 0xc6, 0xc6,
	0xc6, 0xc6, 0xc7, 0xc7, 0xc7, 0xc7, 0xc8, 0xc8,
	0xc8, 0xc8, 0xc9, 0xc9, 0xc9, 0xc9, 0xca, 0xca,
	0xca, 0xcb, 0xcb, 0xcb, 0xcb, 0xcc, 0xcc, 0xcc,
	0xcc, 0xcd, 0xcd, 0xcd, 0xcd, 0xce, 0xce, 0xce,
	0xce, 0xcf, 0xcf, 0xcf, 0xcf, 0xd0, 0xd0, 0xd0,
	0xd0, 0xd1, 0xd1, 0xd1, 0xd1, 0xd2, 0xd2, 0xd2,
	0xd2, 0xd3, 0xd3, 0xd3, 0xd3, 0xd4, 0xd4, 0xd4,
	0xd4, 0xd5, 0xd5, 0xd5, 0xd6, 0xd6, 0xd6, 0xd6,
	0xd7, 0xd7, 0xd7, 0xd7, 0xd8, 0xd8, 0xd8, 0xd8,
	0xd9, 0xd9, 0xd9, 0xd9, 0xda, 0xda, 0xda, 0xda,
	0xdb, 0xdb, 0xdb, 0xdb, 0xdc, 0xdc, 0xdc, 0xdc,
	0xdd, 0xdd, 0xdd, 0xdd, 0xde, 0xde, 0xde, 0xde,
	0xdf, 0xdf, 0xdf, 0xdf, 0xe0, 0xe0, 0xe0, 0xe1,
	0xe1, 0xe1, 0xe1, 0xe2, 0xe2, 0xe2, 0xe2, 0xe3,
	0xe3, 0xe3, 0xe3, 0xe4, 0xe4, 0xe4, 0xe4, 0xe5,
	0xe5, 0xe5, 0xe5, 0xe6, 0xe6, 0xe6, 0xe6, 0xe7,
	0xe7, 0xe7, 0xe7, 0xe8, 0xe8, 0xe8, 0xe8, 0xe9,
	0xe9, 0xe9, 0xe9, 0xea, 0xea, 0xea, 0xea, 0xeb,
	0xeb, 0xeb, 0xec, 0xec, 0xec, 0xec, 0xed, 0xed,
	0xed, 0xed, 0xee, 0xee, 0xee, 0xee, 0xef, 0xef,
	0xef, 0xef, 0xf0, 0xf0, 0xf0, 0xf0, 0xf1, 0xf1,
	0xf1, 0xf1, 0xf2, 0xf2, 0xf2, 0xf2, 0xf3, 0xf3,
	0xf3, 0xf3, 0xf4, 0xf4, 0xf4, 0xf4, 0xf5, 0xf5,
	0xf5, 0xf6, 0xf6, 0xf6, 0xf6, 0xf7, 0xf7, 0xf7,
	0xf7, 0xf8, 0xf8, 0xf8, 0xf8, 0xf9, 0xf9, 0xf9,
	0xf9, 0xfa, 0xfa, 0xfa, 0xfa, 0xfb, 0xfb, 0xfb,
	0xfb, 0xfc, 0xfc, 0xfc, 0xfc, 0xfd, 0xfd, 0xfd,
	0xfd, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff,
	0xff
};
	
/*** ---------------------------------------------------------------------- ***/

_WORD pic_rgb_to_vdi(unsigned char c)
{
	return rgb_to_vdi_tab[c];
}

/*** ---------------------------------------------------------------------- ***/

void pic_rgb_to_vdipal(_WORD *vdi, PALETTE pal, _WORD colors, const _WORD pixel_to_color[])
{
	_WORD i;
	
	for (i = 0; i < colors && i < 256; i++)
	{
		vdi[0] = rgb_to_vdi_tab[pal[i].r];
		vdi[1] = rgb_to_vdi_tab[pal[i].g];
		vdi[2] = rgb_to_vdi_tab[pal[i].b];
		if (pixel_to_color)
		{
			vdi[3] = pixel_to_color[i];
			vdi += 4;
		} else
		{
			vdi += 3;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_vdi_to_rgbpal(PALETTE pal, _WORD *vdi, _WORD colors, _WORD components)
{
	_WORD i;
	_WORD color;
	
	for (i = 0; i < colors && i < 256; i++)
	{
		color = vdi[i * components + 0];
		pal[i].r = vdi_to_rgb_tab[color];
		color = vdi[i * components + 1];
		pal[i].g = vdi_to_rgb_tab[color];
		color = vdi[i * components + 2];
		pal[i].b = vdi_to_rgb_tab[color];
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_vdi_to_rgbcolor(_WORD vdi[3])
{
	_WORD color;
	
	color = vdi[0];
	vdi[0] = vdi_to_rgb_tab[color];
	color = vdi[1];
	vdi[1] = vdi_to_rgb_tab[color];
	color = vdi[2];
	vdi[2] = vdi_to_rgb_tab[color];
}

/*** ---------------------------------------------------------------------- ***/

static void convinit(PALETTE pal)
{
	_WORD i;
	_WORD color;
	
	for (i = 0; i < 16; i++)
	{
		color = (pal[i].r >> 5) & 0x007;
		color += (pal[i].g >> 5) & 0x007;
		color += (pal[i].b >> 5) & 0x007;
		color = ((color * 3) & 0x3c) << 2;
		patptr1[i] = &pattern1[color];
		patptr2[i] = &pattern2[color];
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_16to2(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	_WORD lines;
	_WORD planes;
	_WORD words;
	unsigned char p1, p2, p3, p4;
	unsigned char *line0;
	unsigned char *line1;
	_WORD dots;
	_WORD pix;
	const unsigned char *pat;
	const unsigned char *mask;

	convinit(pic->pi_palette);

	bytes <<= 1;
	pic->pi_width <<= 1;
	pic->pi_height = height << 1;
	pic->pi_planes = 1;
	line0 = dest;
	line1 = dest + bytes;
	for (lines = height, planes = 0; --lines >= 0; )
	{
		for (words = (bytes + 2) >> 2; --words >= 0; )
		{
			p1 = src[0];
			p2 = src[2];
			p3 = src[4];
			p4 = src[6];
			*line0 = 0;
			*line1 = 0;
			for (dots = 8, mask = masktab; --dots >= 0; )
			{
				pix = 0;
				if (p4 & 0x80) pix++;
				p4 += p4;
				pix += pix;
				if (p3 & 0x80) pix++;
				p3 += p3;
				pix += pix;
				if (p2 & 0x80) pix++;
				p2 += p2;
				pix += pix;
				if (p1 & 0x80) pix++;
				p1 += p1;
				pat = patptr1[pix] + planes;
				*line0 |= (*pat++ & *mask);
				*line1 |= (*pat & *mask++);
				if (dots == 4)
				{
					line0++;
					line1++;
					*line0 = 0;
					*line1 = 0;
				}
			}
			line0++;
			line1++;

			if (words != 0 || !(bytes & 2))
			{
				p1 = src[1];
				p2 = src[3];
				p3 = src[5];
				p4 = src[7];
				*line0 = 0;
				*line1 = 0;
				for (dots = 8, mask = masktab; --dots >= 0; )
				{
					pix = 0;
					if (p4 & 0x80) pix++;
					p4 += p4;
					pix += pix;
					if (p3 & 0x80) pix++;
					p3 += p3;
					pix += pix;
					if (p2 & 0x80) pix++;
					p2 += p2;
					pix += pix;
					if (p1 & 0x80) pix++;
					p1 += p1;
					pat = patptr2[pix] + planes;
					*line0 |= (*pat++ & *mask);
					*line1 |= (*pat & *mask++);
					if (dots == 4)
					{
						line0++;
						line1++;
						*line0 = 0;
						*line1 = 0;
					}
				}
				line0++;
				line1++;
			}

			src += 8;
		}

		line0 += bytes;
		line1 += bytes;
		planes += 2;
		if (planes == 16)
			planes = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_4to2(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	_WORD lines;
	_WORD planes;
	_WORD words;
	_WORD p1, p2;
	unsigned char *line0;
	unsigned char *line1;
	_UWORD mask;
	_WORD pix;
	const unsigned char *pat;

	convinit(pic->pi_palette);

	pic->pi_height = height << 1;
	pic->pi_planes = 1;
	bytes = (bytes + 1) & ~1;
	line0 = dest;
	line1 = dest + bytes;
	for (lines = height, planes = 0; --lines >= 0; )
	{
		for (words = bytes >> 1; --words >= 0; )
		{
			p1 = src[0];
			p2 = src[2];
			*line0 = 0;
			*line1 = 0;
			for (mask = 0x80; mask != 0; mask >>= 1)
			{
				pix = 0;
				if (p2 & 0x80) pix++;
				p2 += p2;
				pix += pix;
				if (p1 & 0x80) pix++;
				p1 += p1;
				pat = patptr1[pix] + planes;
				*line0 |= (*pat++ & mask);
				*line1 |= (*pat & mask);
			}
			line0++;
			line1++;

			p1 = src[1];
			p2 = src[3];
			*line0 = 0;
			*line1 = 0;
			for (mask = 0x80; mask != 0; mask >>= 1)
			{
				pix = 0;
				if (p2 & 0x80) pix++;
				p2 += p2;
				pix += pix;
				if (p1 & 0x80) pix++;
				p1 += p1;
				pat = patptr2[pix] + planes;
				*line0 |= (*pat++ & mask);
				*line1 |= (*pat & mask);
			}
			line0++;
			line1++;
			src += 4;
		}
		line0 += bytes;
		line1 += bytes;
		planes += 2;
		if (planes == 16)
			planes = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_16to4(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	_WORD lines;
	_WORD planes;
	_WORD words;
	unsigned char p1, p2, p3, p4;
	unsigned char *line0;
	_WORD dots;
	_WORD pix;
	const unsigned char *pat;
	const unsigned char *mask;

	convinit(pic->pi_palette);
	pic->pi_palette[0].r = 0x00;
	pic->pi_palette[0].g = 0x00;
	pic->pi_palette[0].b = 0x00;
	pic->pi_palette[1].r = 0x80;
	pic->pi_palette[1].g = 0x80;
	pic->pi_palette[1].b = 0x80;
	pic->pi_palette[2].r = 0x80;
	pic->pi_palette[2].g = 0x80;
	pic->pi_palette[2].b = 0x80;
	pic->pi_palette[3].r = 0xe0;
	pic->pi_palette[3].g = 0xe0;
	pic->pi_palette[3].b = 0xe0;

	pic->pi_width <<= 1;
	pic->pi_planes = 2;

	line0 = dest;
	for (lines = height, planes = 0; --lines >= 0; )
	{
		for (words = bytes >> 1; --words >= 0; )
		{
			p1 = src[0];
			p2 = src[2];
			p3 = src[4];
			p4 = src[6];
			line0[0] = 0;
			line0[2] = 0;
			for (dots = 8, mask = masktab; --dots >= 0; )
			{
				pix = 0;
				if (p4 & 0x80) pix++;
				p4 += p4;
				pix += pix;
				if (p3 & 0x80) pix++;
				p3 += p3;
				pix += pix;
				if (p2 & 0x80) pix++;
				p2 += p2;
				pix += pix;
				if (p1 & 0x80) pix++;
				p1 += p1;
				pat = patptr1[pix] + planes;
				line0[0] |= (*pat++ & *mask);
				line0[2] |= (*pat & *mask++);
				if (dots == 4)
				{
					line0++;
					line0[0] = 0;
					line0[2] = 0;
				}
			}

			p1 = src[1];
			p2 = src[3];
			p3 = src[5];
			p4 = src[7];
			line0 += 3;
			line0[0] = 0;
			line0[2] = 0;
			for (dots = 8, mask = masktab; --dots >= 0; )
			{
				pix = 0;
				if (p4 & 0x80) pix++;
				p4 += p4;
				pix += pix;
				if (p3 & 0x80) pix++;
				p3 += p3;
				pix += pix;
				if (p2 & 0x80) pix++;
				p2 += p2;
				pix += pix;
				if (p1 & 0x80) pix++;
				p1 += p1;
				pat = patptr2[pix] + planes;
				line0[0] |= (*pat++ & *mask);
				line0[2] |= (*pat & *mask++);
				if (dots == 4)
				{
					line0++;
					line0[0] = 0;
					line0[2] = 0;
				}
			}
			line0 += 3;
			src += 8;
		}
		planes += 2;
		if (planes == 16)
			planes = 0;
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_2to16(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	_WORD words;
	unsigned char p1;
	unsigned char *line0;
	unsigned char *line1;
	unsigned char pix1, pix2;
	_WORD mask;

	convinit(pic->pi_palette);
	pic_getpalette(pic->pi_palette, &palet1);
	bytes = (bytes + 1) & ~1;
	line0 = src;
	line1 = src + bytes;

	height >>= 1;
	bytes >>= 1;
	pic->pi_height = height;
	pic->pi_width >>= 1;
	pic->pi_planes = 4;
	for (; height > 0; height--)
	{
		for (words = bytes; --words >= 0; )
		{
			pix1 = pix2 = 0;
			p1 = *line0++;
			for (mask = 4; --mask >= 0; )
			{
				pix1 += pix1;
				if (p1 & 0x80) pix1++;
				p1 += p1;
				pix2 += pix2;
				if (p1 & 0x80) pix2++;
				p1 += p1;
			}
			p1 = *line0++;
			for (mask = 4; --mask >= 0; )
			{
				pix1 += pix1;
				if (p1 & 0x80) pix1++;
				p1 += p1;
				pix2 += pix2;
				if (p1 & 0x80) pix2++;
				p1 += p1;
			}
			dest[0] = pix1;
			dest[2] = pix2;
			pix1 = pix2 = 0;
			if (words != 0)
			{
				p1 = *line0++;
				for (mask = 4; --mask >= 0; )
				{
					pix1 += pix1;
					if (p1 & 0x80) pix1++;
					p1 += p1;
					pix2 += pix2;
					if (p1 & 0x80) pix2++;
					p1 += p1;
				}
				p1 = *line0++;
				for (mask = 4; --mask >= 0; )
				{
					pix1 += pix1;
					if (p1 & 0x80) pix1++;
					p1 += p1;
					pix2 += pix2;
					if (p1 & 0x80) pix2++;
					p1 += p1;
				}
			}
			dest[1] = pix1;
			dest[3] = pix2;
			dest += 4;

			pix1 = pix2 = 0;
			p1 = *line1++;
			for (mask = 4; --mask >= 0; )
			{
				pix1 += pix1;
				if (p1 & 0x80) pix1++;
				p1 += p1;
				pix2 += pix2;
				if (p1 & 0x80) pix2++;
				p1 += p1;
			}
			p1 = *line1++;
			for (mask = 4; --mask >= 0; )
			{
				pix1 += pix1;
				if (p1 & 0x80) pix1++;
				p1 += p1;
				pix2 += pix2;
				if (p1 & 0x80) pix2++;
				p1 += p1;
			}
			dest[0] = pix1;
			dest[2] = pix2;
			pix1 = pix2 = 0;
			if (words != 0)
			{
				--words;
				p1 = *line1++;
				for (mask = 4; --mask >= 0; )
				{
					pix1 += pix1;
					if (p1 & 0x80) pix1++;
					p1 += p1;
					pix2 += pix2;
					if (p1 & 0x80) pix2++;
					p1 += p1;
				}
				p1 = *line1++;
				for (mask = 4; --mask >= 0; )
				{
					pix1 += pix1;
					if (p1 & 0x80) pix1++;
					p1 += p1;
					pix2 += pix2;
					if (p1 & 0x80) pix2++;
					p1 += p1;
				}
			}
			dest[1] = pix1;
			dest[3] = pix2;
			dest += 4;
		}
		line0 = line0 + bytes + bytes;
		line1 = line1 + bytes + bytes;
	}

}

/*** ---------------------------------------------------------------------- ***/

void pic_2to4(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	_WORD lines;
	_WORD words;
	unsigned char *line0;
	unsigned char *line1;

	convinit(pic->pi_palette);
	pic->pi_palette[0].r = 0xe0;
	pic->pi_palette[0].g = 0xe0;
	pic->pi_palette[0].b = 0xe0;
	pic->pi_palette[1].r = 0x80;
	pic->pi_palette[1].g = 0x80;
	pic->pi_palette[1].b = 0x80;
	pic->pi_palette[2].r = 0x80;
	pic->pi_palette[2].g = 0x80;
	pic->pi_palette[2].b = 0x80;
	pic->pi_palette[3].r = 0x00;
	pic->pi_palette[3].g = 0x00;
	pic->pi_palette[3].b = 0x00;

	height >>= 1;
	pic->pi_height = height;
	pic->pi_planes = 2;
	bytes = (bytes + 1) & ~1;
	line0 = src;
	line1 = line0 + bytes;
	for (lines = height; --lines >= 0; )
	{
		for (words = bytes >> 1; --words >= 0; )
		{
			*dest++ = *line0++;
			*dest++ = *line0++;
			*dest++ = *line1++;
			*dest++ = *line1++;
		}
		line0 += bytes;
		line1 += bytes;
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_4to16(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	_WORD lines;
	_WORD words;
	unsigned char p1, p2, p3, p4;
	_WORD dots;
	_WORD pix1, pix2;

	convinit(pic->pi_palette);

	pic->pi_width >>= 1;
	pic->pi_planes = 4;
	bytes >>= 1;
	for (lines = height; --lines >= 0; )
	{
		for (words = bytes >> 1; --words >= 0; )
		{
			p1 = p2 = p3 = p4 = 0;
			pix2 = src[0];
			pix1 = src[2];
			for (dots = 4; --dots >= 0; )
			{
				p1 += p1;
				if (pix1 & 0x80) p1++;
				pix1 += pix1;
				p2 += p2;
				if (pix2 & 0x80) p2++;
				pix2 += pix2;
				p3 += p3;
				if (pix1 & 0x80) p3++;
				pix1 += pix1;
				p4 += p4;
				if (pix2 & 0x80) p4++;
				pix2 += pix2;
			}
			pix2 = src[1];
			pix1 = src[3];
			for (dots = 4; --dots >= 0; )
			{
				p1 += p1;
				if (pix1 & 0x80) p1++;
				pix1 += pix1;
				p2 += p2;
				if (pix2 & 0x80) p2++;
				pix2 += pix2;
				p3 += p3;
				if (pix1 & 0x80) p3++;
				pix1 += pix1;
				p4 += p4;
				if (pix2 & 0x80) p4++;
				pix2 += pix2;
			}
			dest[0] = p1;
			dest[2] = p2;
			dest[4] = p3;
			dest[6] = p4;

			p1 = p2 = p3 = p4 = 0;
			pix2 = src[4];
			pix1 = src[6];
			for (dots = 4; --dots >= 0; )
			{
				p1 += p1;
				if (pix1 & 0x80) p1++;
				pix1 += pix1;
				p2 += p2;
				if (pix2 & 0x80) p2++;
				pix2 += pix2;
				p3 += p3;
				if (pix1 & 0x80) p3++;
				pix1 += pix1;
				p4 += p4;
				if (pix2 & 0x80) p4++;
				pix2 += pix2;
			}
			pix2 = src[5];
			pix1 = src[7];
			for (dots = 4; --dots >= 0; )
			{
				p1 += p1;
				if (pix1 & 0x80) p1++;
				pix1 += pix1;
				p2 += p2;
				if (pix2 & 0x80) p2++;
				pix2 += pix2;
				p3 += p3;
				if (pix1 & 0x80) p3++;
				pix1 += pix1;
				p4 += p4;
				if (pix2 & 0x80) p4++;
				pix2 += pix2;
			}
			dest[1] = p1;
			dest[3] = p2;
			dest[5] = p3;
			dest[7] = p4;

			src += 8;
			dest += 8;
		}
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_getpalette(PALETTE pal, const TOSPALETTE *tospal)
{
	_WORD i;

	for (i = 0; i < 16; i++)
	{
		pal[i].r = ((((*tospal)[i][0]     ) & 0x07) << 1) | ((((*tospal)[i][0] >> 3) & 0x01) << 0);
		pal[i].r = (pal[i].r << 4) | pal[i].r;
		pal[i].g = ((((*tospal)[i][1] >> 4) & 0x07) << 1) | ((((*tospal)[i][1] >> 7) & 0x01) << 0);
		pal[i].g = (pal[i].g << 4) | pal[i].g;
		pal[i].b = ((((*tospal)[i][1]     ) & 0x07) << 1) | ((((*tospal)[i][1] >> 3) & 0x01) << 0);
		pal[i].b = (pal[i].b << 4) | pal[i].b;
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_setpalette(TOSPALETTE tospal, PALETTE pal)
{
	_WORD i;

	for (i = 0; i < 16; i++)
	{
		tospal[i][0] = (((pal[i].r >> 5) & 0x07)     ) | (((pal[i].r >> 4) & 0x01) << 3);
		tospal[i][1] = (((pal[i].g >> 5) & 0x07) << 4) | (((pal[i].g >> 4) & 0x01) << 7) |
					   (((pal[i].b >> 5) & 0x07)     ) | (((pal[i].b >> 4) & 0x01) << 3);
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_stdpalette(PALETTE pal, _WORD planes)
{
	_WORD i;
	_WORD colors;
	
	if (planes > 8)
		planes = 8;
	colors = 1 << planes;
	for (i = 0; i < colors; i++)
		pal[i] = std256_palette[i];
	if (planes < 8)
	{
		pal[colors - 1] = std256_palette[255];
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_savepalette(PALETTE pal)
{
#if defined(__TOS__) || defined(__atarist__)
	TOSPALETTE tospal;
	_WORD *palp = (_WORD *)(tospal);
	int i;
	
	for (i = 0; i < 16; i++)
		palp[i] = Setcolor(i, -1);
	pic_getpalette(pal, (const TOSPALETTE *)&tospal);
#else
	pic_stdpalette(pal, 1);
#endif
}

/*** ---------------------------------------------------------------------- ***/

void pic_showpalette(PALETTE pal)
{
#if (defined(__TOS__) || defined(__atarist__)) && 0
	TOSPALETTE tospal;
	int i;
	_WORD *palp = (_WORD *)(tospal);
	
	pic_setpalette(tospal, pal);
	for (i = 0; i < 16; i++)
		(void) Setcolor(i, palp[i]);
	Vsync();
	pic_getpalette(pal, (const TOSPALETTE *)&tospal);
#else
	UNUSED(pal);
#endif
}

/*** ---------------------------------------------------------------------- ***/

void pic_invert(unsigned char *buf, _LONG size)
{
	while (--size >= 0)
	{
		*buf = ~(*buf);
		buf++;
	}
}

/*** ---------------------------------------------------------------------- ***/

_WORD pic_calcrez(_WORD planes)
{
	return planes == 4 ? 0 : planes == 2 ? 1 : planes == 8 ? 3 : 2;
}

/*** ---------------------------------------------------------------------- ***/

_WORD pic_calcplanes(_WORD rez)
{
	return rez == 0 ? 4 : rez == 1 ? 2 : rez == 3 ? 8 : 1;
}

/*** ---------------------------------------------------------------------- ***/

_LONG pic_rowsize(PICTURE *pic, _WORD planes)
{
	return (_LONG)tobyte(pic->pi_width) * (_LONG)planes;
}

/*** ---------------------------------------------------------------------- ***/

void pic_calcsize(PICTURE *pic)
{
	_LONG bytes = pic_rowsize(pic, pic->pi_planes);
	
	pic->pi_bytes = tobyte(pic->pi_width);
	pic->pi_picsize = bytes * pic->pi_height;
	
	pic->pi_fdb.fd_nplanes = pic->pi_planes;
	pic->pi_fdb.fd_w = pic->pi_width;
	pic->pi_fdb.fd_h = pic->pi_height;
	pic->pi_fdb.fd_wdwidth = toword(pic->pi_width);
	pic->pi_fdb.fd_stand = 0;
	pic->pi_fdb.fd_r1 = 0;
	pic->pi_fdb.fd_r2 = 0;
	pic->pi_fdb.fd_r3 = 0;
}

/*** ---------------------------------------------------------------------- ***/

/*
 * Some formats (like GIF) can have some unusual number of colors.
 * We want do deal only with monochrome, 16 and 256 colors.
 */
void pic_normal_planes(PICTURE *pic)
{
	switch (pic->pi_planes)
	{
	case 1:
		break;
	case 2:
		if (pic->pi_type != FT_DEGAS_MED && pic->pi_type != FT_IMAGIC_MED)
			pic->pi_planes = 4;
		break;
	case 3:
	case 4:
		pic->pi_planes = 4;
		break;
	case 15:
	case 16:
		pic->pi_planes = 16;
		break;
	case 24:
	case 32:
		break;
	default:
		pic->pi_planes = 8;
		break;
	}
	pic_calcsize(pic);
}

/*** ---------------------------------------------------------------------- ***/

void pic_stdsize(PICTURE *pic)
{
	static _WORD const maxwidth[4] = { 320, 640, 640, 640 };
	static _WORD const maxheight[4] = { 200, 200, 400, 480 };
	_WORD rez;

	rez = pic_calcrez(pic->pi_planes);
	pic->pi_orig_planes = pic->pi_planes;
	pic->pi_width = pic->pi_orig_width = maxwidth[rez];
	pic->pi_height = pic->pi_orig_height = maxheight[rez];
	pic_calcsize(pic);
}

/*** ---------------------------------------------------------------------- ***/

static gboolean is_palette(const TOSPALETTE pal)
{
	_WORD i;

	for (i = 0; i < 16; i++)
	{
		if ((pal[i][0] & 0x08) != 0)
			return TRUE;
		if ((pal[i][1] & 0x88) != 0)
			return TRUE;
	}
	return TRUE;
}

/*** ---------------------------------------------------------------------- ***/

typedef struct {                /* Header fuer Neochrome-Bilder */
	unsigned char stuff1[4];
	TOSPALETTE palette;         /* Farbpalette */
	unsigned char stuff2[92];
} NEO_HEADER;

gboolean pic_type_neo(PICTURE *pic, const unsigned char *buf, _LONG size)
{
	const NEO_HEADER *header = (const NEO_HEADER *)buf;

	if (pic->pi_filesize >= 32128l &&
		pic->pi_filesize <= 32768l &&
		size >= 128 &&
		is_palette(header->palette))
	{
		pic->pi_type = FT_NEO;
		pic->pi_planes = 4;
		pic_stdsize(pic);
		pic->pi_datasize = size - sizeof(NEO_HEADER);
		pic->pi_dataoffset = sizeof(NEO_HEADER);
		pic_getpalette(pic->pi_palette, &header->palette);
		pic->pi_compressed = FALSE;
		return TRUE;
	}
	return FALSE;
}

/*** ---------------------------------------------------------------------- ***/

_LONG neo_header(unsigned char *buf, PICTURE *pic)
{
	NEO_HEADER *header = (NEO_HEADER *)buf;

	memset(buf, 0, NEO_HEADER_SIZE);
	pic_setpalette(header->palette, pic->pi_palette);
	pic->pi_datasize = 32000;
	return NEO_HEADER_SIZE;
}

/*** ---------------------------------------------------------------------- ***/

typedef struct {                /* Header fuer Colorstar-Bilder */
	TOSPALETTE palette;         /* Farbpalette */
} COLOR_HEADER;

gboolean pic_type_colorstar(PICTURE *pic, const unsigned char *buf, _LONG size)
{
	const COLOR_HEADER *header = (const COLOR_HEADER *)buf;

	if (pic->pi_filesize == 32032l &&
		size > 32 &&
		is_palette(header->palette))
	{
		pic->pi_type = FT_COLSTAR;
		pic->pi_planes = 4;
		pic_stdsize(pic);
		pic_getpalette(pic->pi_palette, &header->palette);
		pic->pi_datasize = pic->pi_filesize - COLOR_HEADER_SIZE;
		pic->pi_dataoffset = COLOR_HEADER_SIZE;
		pic->pi_compressed = FALSE;
		return TRUE;
	}
	return FALSE;
}


_LONG colstar_header(unsigned char *buf, PICTURE *pic)
{
	COLOR_HEADER *header = (COLOR_HEADER *)buf;

	pic_setpalette(header->palette, pic->pi_palette);
	pic->pi_datasize = 32000l;
	return COLOR_HEADER_SIZE;
}

/*** ---------------------------------------------------------------------- ***/

gboolean pic_type_monostar(PICTURE *pic, const unsigned char *buf, _LONG size)
{
	UNUSED(buf);
	UNUSED(size);
	if (pic->pi_filesize == 32000l)
	{
		pic->pi_type = FT_SCREEN;
		pic->pi_planes = 1;
		pic_stdsize(pic);
		pic->pi_datasize = size;
		pic->pi_dataoffset = 0;
		pic->pi_compressed = FALSE;
		return TRUE;
	}
	return FALSE;
}

_LONG monostar_header(unsigned char *buf, PICTURE *pic)
{
	UNUSED(buf);
	pic->pi_datasize = 32000l;
	return 0;
}

/*** ---------------------------------------------------------------------- ***/

void pic_planes_to_interleaved(unsigned char *dst, const unsigned char *src, PICTURE *pic)
{
	long planesize;
	const _UWORD *row1, *row2, *row3, *row4, *row5, *row6, *row7, *row8;
	_UWORD *p;
	_WORD x, y;
	
	planesize = (_LONG)pic->pi_height * toword(pic->pi_width);
	p = (_UWORD *)dst;
	row1 = (const _UWORD *)src;
	
	switch (pic->pi_planes)
	{
	case 1:
		memcpy(dst, src, planesize << 1);
		break;
	case 2:
		row2 = row1 + planesize;
		for (y = 0; y < pic->pi_height; y++)
		{
			for (x = 0; x < pic->pi_width; x += 16)
			{
				*p++ = *row1++;
				*p++ = *row2++;
			}
		}
		break;
	case 4:
		row2 = row1 + planesize;
		row3 = row2 + planesize;
		row4 = row3 + planesize;
		for (y = 0; y < pic->pi_height; y++)
		{
			for (x = 0; x < pic->pi_width; x += 16)
			{
				*p++ = *row1++;
				*p++ = *row2++;
				*p++ = *row3++;
				*p++ = *row4++;
			}
		}
		break;
	case 8:
		row2 = row1 + planesize;
		row3 = row2 + planesize;
		row4 = row3 + planesize;
		row5 = row4 + planesize;
		row6 = row5 + planesize;
		row7 = row6 + planesize;
		row8 = row7 + planesize;
		for (y = 0; y < pic->pi_height; y++)
		{
			for (x = 0; x < pic->pi_width; x += 16)
			{
				*p++ = *row1++;
				*p++ = *row2++;
				*p++ = *row3++;
				*p++ = *row4++;
				*p++ = *row5++;
				*p++ = *row6++;
				*p++ = *row7++;
				*p++ = *row8++;
			}
		}
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

void pic_interleaved_to_planes(unsigned char *dst, const unsigned char *src, _WORD width, _WORD height, _WORD planes)
{
	long planesize;
	_UWORD *row1, *row2, *row3, *row4, *row5, *row6, *row7, *row8;
	const _UWORD *p;
	_WORD x, y;
	
	planesize = (_LONG)height * toword(width);
	p = (const _UWORD *)src;
	row1 = (_UWORD *)dst;
	
	switch (planes)
	{
	case 1:
		memcpy(dst, src, planesize << 1);
		break;
	case 2:
		row2 = row1 + planesize;
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < width; x += 16)
			{
				*row1++ = *p++;
				*row2++ = *p++;
			}
		}
		break;
	case 4:
		row2 = row1 + planesize;
		row3 = row2 + planesize;
		row4 = row3 + planesize;
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < width; x += 16)
			{
				*row1++ = *p++;
				*row2++ = *p++;
				*row3++ = *p++;
				*row4++ = *p++;
			}
		}
		break;
	case 8:
		row2 = row1 + planesize;
		row3 = row2 + planesize;
		row4 = row3 + planesize;
		row5 = row4 + planesize;
		row6 = row5 + planesize;
		row7 = row6 + planesize;
		row8 = row7 + planesize;
		for (y = 0; y < height; y++)
		{
			for (x = 0; x < width; x += 16)
			{
				*row1++ = *p++;
				*row2++ = *p++;
				*row3++ = *p++;
				*row4++ = *p++;
				*row5++ = *p++;
				*row6++ = *p++;
				*row7++ = *p++;
				*row8++ = *p++;
			}
		}
		break;
	}
}

/*** ---------------------------------------------------------------------- ***/

pic_filetype pic_type(PICTURE *pic, unsigned char *buf, _LONG size)
{
	pic_filetype type;

	type = FT_UNKNOWN;
	pic->pi_filesize = size;
	if (pic_type_iff(pic, buf, size) ||
		pic_type_gif(pic, buf, size) ||
		pic_type_img(pic, buf, size) ||
		pic_type_imagic(pic, buf, size) ||
		pic_type_stad(pic, buf, size) ||
		pic_type_colorstar(pic, buf, size) ||
		pic_type_bmp(pic, buf, size) ||
		pic_type_ico(pic, buf, size) ||
		pic_type_monostar(pic, buf, size) ||
		pic_type_degas(pic, buf, size) ||
		pic_type_neo(pic, buf, size) ||
		pic_type_icn(pic, buf, size) ||
		pic_type_png(pic, buf, size))
	{
		type = pic->pi_type;
	} else
	{
	}

	pic->pi_orig_planes = pic->pi_planes;
	pic->pi_orig_width = pic->pi_width;
	pic->pi_orig_height = pic->pi_height;
	return type;
}

/*** ---------------------------------------------------------------------- ***/

void pic_init(PICTURE *pic)
{
	memset(pic, 0, sizeof(*pic));
	pic->pi_planes = 1;
	pic_stdsize(pic);
	pic_savepalette(pic->pi_palette);
	pic->pi_datasize = 0;
	pic->pi_compressed = 0;
	pic->pi_pat_len = 2;
	pic->pi_dataoffset = 0;
	pic->pi_version = 1;
	pic->pi_pix_width = 0;
	pic->pi_pix_height = 0;
	pic->pi_serial = 0;
	pic->pi_type = FT_UNKNOWN;
	pic->pi_filesize = 0;

	pic->pi_snap.g_x =
	pic->pi_snap.g_y =
	pic->pi_snap.g_w =
	pic->pi_snap.g_h = 0;
	pic->pi_buf = NULL;
	pic->pi_active = FALSE;
	pic->pi_unsupported = FALSE;
	pic->pi_topdown = TRUE;
	pic->pi_transparent = -1;
#ifdef IN_ORCS
	pic->pi_name[0] = '\0';
#else
	pic->pi_name = NULL;
#endif
}


void pic_free(PICTURE *pic)
{
	if (pic)
	{
		g_free(pic->pi_buf),
		pic->pi_buf = NULL;
	}
}


void pic_256to2(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	UNUSED(dest);
	UNUSED(src);
	UNUSED(pic);
	UNUSED(bytes);
	UNUSED(height);
}
void pic_256to4(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	UNUSED(dest);
	UNUSED(src);
	UNUSED(pic);
	UNUSED(bytes);
	UNUSED(height);
}
void pic_256to16(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	UNUSED(dest);
	UNUSED(src);
	UNUSED(pic);
	UNUSED(bytes);
	UNUSED(height);
}
void pic_16to256(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	UNUSED(dest);
	UNUSED(src);
	UNUSED(pic);
	UNUSED(bytes);
	UNUSED(height);
}
void pic_4to256(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	UNUSED(dest);
	UNUSED(src);
	UNUSED(pic);
	UNUSED(bytes);
	UNUSED(height);
}
void pic_2to256(unsigned char *dest, unsigned char *src, PICTURE *pic, _WORD bytes, _WORD height)
{
	UNUSED(dest);
	UNUSED(src);
	UNUSED(pic);
	UNUSED(bytes);
	UNUSED(height);
}
